# Impute
[![stable](https://img.shields.io/badge/docs-stable-blue.svg)](https://invenia.github.io/Impute.jl/stable/)
[![latest](https://img.shields.io/badge/docs-latest-blue.svg)](https://invenia.github.io/Impute.jl/latest/)
[![Build Status](https://travis-ci.org/invenia/Impute.jl.svg?branch=master)](https://travis-ci.org/invenia/Impute.jl)
[![Build status](https://ci.appveyor.com/api/projects/status/github/invenia/Impute.jl?svg=true)](https://ci.appveyor.com/project/invenia/Impute-jl)
[![codecov](https://codecov.io/gh/invenia/Impute.jl/branch/master/graph/badge.svg)](https://codecov.io/gh/invenia/Impute.jl)

Impute.jl provides various data imputation methods for `Arrays` and `DataFrames` with various types of missing values.

## Installation
```julia
Pkg.clone("https://github.com/invenia/Impute.jl")
```

## Features

* Operating over Vectors, Matrices and DataFrames
* Chaining of methods

## Methods

* drop - remove missing
* locf - last observation carried forward
* nocb - next observation carried backward
* interp - linear interpolation of values in vector
* fill - replace with a specific value or a function which returns a value given the existing vector with missing values dropped.

## Quickstart

We'll start by imputing `NaN` values in 1-dimension vector.
```julia
julia> using Impute

julia> a = collect(1.0:1.0:20.0)
20-element Array{Float64,1}:
  1.0
  2.0
  3.0
  4.0
  5.0
  6.0
  7.0
  8.0
  9.0
 10.0
 11.0
 12.0
 13.0
 14.0
 15.0
 16.0
 17.0
 18.0
 19.0
 20.0

julia> a[[2, 3, 7]] = NaN
NaN
```

The most common approach to missing data is to remove them.
```julia
julia> impute(a, :drop; limit=0.2)
17-element Array{Float64,1}:
  1.0
  4.0
  5.0
  6.0
  8.0
  9.0
 10.0
 11.0
 12.0
 13.0
 14.0
 15.0
 16.0
 17.0
 18.0
 19.0
 20.0
```

But we may want use linear interpolation, filling, etc
```julia
julia> impute(a, :interp; limit=0.2)
20-element Array{Float64,1}:
  1.0
  2.0
  3.0
  4.0
  5.0
  6.0
  7.0
  8.0
  9.0
 10.0
 11.0
 12.0
 13.0
 14.0
 15.0
 16.0
 17.0
 18.0
 19.0
 20.0

julia> impute(a, :fill; limit=0.2)
20-element Array{Float64,1}:
  1.0
 11.6471
 11.6471
  4.0
  5.0
  6.0
 11.6471
  8.0
  9.0
 10.0
 11.0
 12.0
 13.0
 14.0
 15.0
 16.0
 17.0
 18.0
 19.0
 20.0

julia> impute(a, :locf; limit=0.2)
20-element Array{Float64,1}:
  1.0
  1.0
  1.0
  4.0
  5.0
  6.0
  6.0
  8.0
  9.0
 10.0
 11.0
 12.0
 13.0
 14.0
 15.0
 16.0
 17.0
 18.0
 19.0
 20.0

julia> impute(a, :nocb; limit=0.2)
20-element Array{Float64,1}:
  1.0
  4.0
  4.0
  4.0
  5.0
  6.0
  8.0
  8.0
  9.0
 10.0
 11.0
 12.0
 13.0
 14.0
 15.0
 16.0
 17.0
 18.0
 19.0
 20.0
```

We can also perform these operations on `DataFrame`s.

```julia
julia> using DataFrames

julia> using RDatasets

julia> df = dataset("boot", "neuro")
2814
Symbol[:V1,:V2,:V3,:V4,:V5,:V6]
6
469×6 DataFrames.DataFrame
│ Row │ V1     │ V2     │ V3     │ V4    │ V5    │ V6    │
├─────┼────────┼────────┼────────┼───────┼───────┼───────┤
│ 1   │ NA     │ -203.7 │ -84.1  │ 18.5  │ NA    │ NA    │
│ 2   │ NA     │ -203.0 │ -97.8  │ 25.8  │ 134.7 │ NA    │
│ 3   │ NA     │ -249.0 │ -92.1  │ 27.8  │ 177.1 │ NA    │
│ 4   │ NA     │ -231.5 │ -97.5  │ 27.0  │ 150.3 │ NA    │
│ 5   │ NA     │ NA     │ -130.1 │ 25.8  │ 160.0 │ NA    │
│ 6   │ NA     │ -223.1 │ -70.7  │ 62.1  │ 197.5 │ NA    │
│ 7   │ NA     │ -164.8 │ -12.2  │ 76.8  │ 202.8 │ NA    │
│ 8   │ NA     │ -221.6 │ -81.9  │ 27.5  │ 144.5 │ NA    │
│ 9   │ NA     │ -153.7 │ -17.0  │ 76.1  │ 222.4 │ NA    │
│ 10  │ NA     │ -184.7 │ -47.3  │ 74.4  │ 208.9 │ NA    │
│ 11  │ NA     │ NA     │ -148.8 │ 11.4  │ 137.7 │ NA    │
│ 12  │ NA     │ -197.6 │ -6.4   │ 137.1 │ NA    │ NA    │
│ 13  │ NA     │ -247.8 │ -35.4  │ 80.9  │ 229.5 │ NA    │
│ 14  │ NA     │ -227.0 │ -104.7 │ 20.2  │ 140.2 │ NA    │
│ 15  │ -233.6 │ -115.9 │ -10.5  │ 70.0  │ 202.6 │ NA    │
│ 16  │ NA     │ -232.4 │ -100.6 │ 16.8  │ 145.1 │ NA    │
│ 17  │ NA     │ -199.4 │ -58.2  │ 29.1  │ 184.4 │ NA    │
│ 18  │ NA     │ -195.7 │ -89.5  │ 26.4  │ 142.7 │ NA    │
│ 19  │ NA     │ -180.1 │ -65.0  │ 27.3  │ 171.1 │ NA    │
│ 20  │ NA     │ NA     │ -85.2  │ 27.1  │ NA    │ NA    │
│ 21  │ NA     │ -217.3 │ -77.1  │ 27.6  │ 151.5 │ NA    │
│ 22  │ NA     │ -139.7 │ -15.8  │ 83.0  │ 215.5 │ NA    │
│ 23  │ -249.6 │ -132.8 │ -14.1  │ 78.1  │ 205.7 │ NA    │
│ 24  │ NA     │ -152.7 │ -36.9  │ 29.7  │ 149.8 │ NA    │
│ 25  │ NA     │ -224.1 │ -81.9  │ 29.1  │ 172.2 │ NA    │
│ 26  │ NA     │ NA     │ -235.8 │ 6.0   │ 144.4 │ NA    │
│ 27  │ NA     │ -202.8 │ -45.1  │ 84.0  │ 227.3 │ NA    │
│ 28  │ -240.9 │ -138.4 │ -21.5  │ 73.4  │ 210.6 │ NA    │
│ 29  │ -247.1 │ -128.2 │ -31.3  │ 29.2  │ 143.1 │ NA    │
│ 30  │ NA     │ -185.4 │ -80.3  │ 23.9  │ 115.8 │ 222.7 │
│ 31  │ NA     │ -182.5 │ -75.8  │ 27.5  │ 165.2 │ NA    │
│ 32  │ NA     │ -202.2 │ -99.1  │ 23.8  │ 136.3 │ 242.5 │
│ 33  │ NA     │ -193.3 │ -82.6  │ 26.3  │ 160.5 │ NA    │
│ 34  │ NA     │ -189.4 │ -63.3  │ 27.6  │ 136.8 │ NA    │
│ 35  │ NA     │ -149.0 │ -31.0  │ 73.5  │ 187.8 │ NA    │
│ 36  │ NA     │ -162.4 │ -26.5  │ 72.6  │ NA    │ NA    │
│ 37  │ NA     │ -213.4 │ -107.2 │ 24.7  │ 158.5 │ NA    │
⋮
│ 432 │ NA     │ -156.2 │ -32.9  │ 63.3  │ 182.8 │ NA    │
│ 433 │ NA     │ -220.6 │ -114.2 │ 9.7   │ 106.4 │ 227.9 │
│ 434 │ -219.9 │ -120.9 │ -1.3   │ 99.5  │ 207.6 │ NA    │
│ 435 │ NA     │ -240.5 │ -110.3 │ 26.1  │ 142.8 │ NA    │
│ 436 │ NA     │ -239.6 │ -121.4 │ 2.9   │ 124.9 │ NA    │
│ 437 │ NA     │ -139.8 │ -7.3   │ 121.0 │ NA    │ NA    │
│ 438 │ NA     │ -212.0 │ -66.2  │ 50.4  │ 178.2 │ NA    │
│ 439 │ NA     │ -232.7 │ -109.2 │ 18.4  │ 127.5 │ NA    │
│ 440 │ NA     │ -236.3 │ -115.1 │ 5.1   │ 109.0 │ 212.0 │
│ 441 │ -241.2 │ -107.1 │ -9.1   │ 95.1  │ 198.6 │ NA    │
│ 442 │ -226.7 │ -143.8 │ -30.4  │ 75.8  │ 196.6 │ NA    │
│ 443 │ NA     │ -131.8 │ -26.5  │ 64.7  │ 177.2 │ NA    │
│ 444 │ NA     │ -144.9 │ -0.9   │ 105.3 │ 230.9 │ NA    │
│ 445 │ NA     │ -214.0 │ -81.8  │ 66.1  │ 191.3 │ NA    │
│ 446 │ NA     │ -210.6 │ -94.3  │ 16.7  │ 125.5 │ 239.7 │
│ 447 │ -215.8 │ -114.8 │ -18.4  │ 65.3  │ 171.6 │ 249.7 │
│ 448 │ NA     │ -156.0 │ -14.0  │ 113.7 │ 249.3 │ NA    │
│ 449 │ NA     │ -210.5 │ -41.9  │ NA    │ NA    │ NA    │
│ 450 │ NA     │ -189.2 │ -72.0  │ 56.8  │ 133.8 │ 246.7 │
│ 451 │ NA     │ -214.2 │ -102.2 │ 5.5   │ 75.6  │ 154.3 │
│ 452 │ -219.6 │ -107.9 │ -16.0  │ 101.7 │ 186.0 │ NA    │
│ 453 │ NA     │ -153.0 │ -38.0  │ 61.3  │ 144.4 │ 245.9 │
│ 454 │ NA     │ -179.8 │ -63.4  │ 56.0  │ 157.5 │ NA    │
│ 455 │ NA     │ -174.5 │ -44.8  │ 73.3  │ 179.7 │ NA    │
│ 456 │ NA     │ -206.8 │ -108.9 │ 3.7   │ 102.1 │ 210.3 │
│ 457 │ NA     │ -169.5 │ -79.7  │ 27.9  │ 129.4 │ 242.8 │
│ 458 │ -222.2 │ -104.6 │ -2.4   │ 84.3  │ 204.7 │ NA    │
│ 459 │ -236.3 │ -124.0 │ -6.8   │ 95.7  │ 196.0 │ NA    │
│ 460 │ NA     │ -216.5 │ -90.2  │ 27.8  │ 138.9 │ NA    │
│ 461 │ NA     │ -163.2 │ -43.6  │ 69.5  │ 173.9 │ NA    │
│ 462 │ NA     │ -207.3 │ -88.3  │ 9.6   │ 104.1 │ 218.0 │
│ 463 │ -242.6 │ -142.0 │ -21.8  │ 69.8  │ 148.7 │ NA    │
│ 464 │ -235.9 │ -128.8 │ -33.1  │ 68.8  │ 177.1 │ NA    │
│ 465 │ NA     │ -140.8 │ -38.7  │ 58.1  │ 186.3 │ NA    │
│ 466 │ NA     │ -149.5 │ -40.3  │ 62.8  │ 139.7 │ 242.5 │
│ 467 │ -247.6 │ -157.8 │ -53.3  │ 28.3  │ 122.9 │ 227.6 │
│ 468 │ NA     │ -154.9 │ -50.8  │ 28.1  │ 119.9 │ 201.1 │
│ 469 │ NA     │ -180.7 │ -70.9  │ 33.7  │ 114.8 │ 222.5 │

julia> drop(df)
4×6 DataFrames.DataFrame
│ Row │ V1     │ V2     │ V3    │ V4   │ V5    │ V6    │
├─────┼────────┼────────┼───────┼──────┼───────┼───────┤
│ 1   │ -247.0 │ -132.2 │ -18.8 │ 28.2 │ 81.4  │ 237.9 │
│ 2   │ -234.0 │ -140.8 │ -56.5 │ 28.0 │ 114.3 │ 222.9 │
│ 3   │ -215.8 │ -114.8 │ -18.4 │ 65.3 │ 171.6 │ 249.7 │
│ 4   │ -247.6 │ -157.8 │ -53.3 │ 28.3 │ 122.9 │ 227.6 │

julia> interp(df)
469×6 DataFrames.DataFrame
│ Row │ V1       │ V2      │ V3     │ V4    │ V5     │ V6      │
├─────┼──────────┼─────────┼────────┼───────┼────────┼─────────┤
│ 1   │ NA       │ -203.7  │ -84.1  │ 18.5  │ NA     │ NA      │
│ 2   │ NA       │ -203.0  │ -97.8  │ 25.8  │ 134.7  │ NA      │
│ 3   │ NA       │ -249.0  │ -92.1  │ 27.8  │ 177.1  │ NA      │
│ 4   │ NA       │ -231.5  │ -97.5  │ 27.0  │ 150.3  │ NA      │
│ 5   │ NA       │ -227.3  │ -130.1 │ 25.8  │ 160.0  │ NA      │
│ 6   │ NA       │ -223.1  │ -70.7  │ 62.1  │ 197.5  │ NA      │
│ 7   │ NA       │ -164.8  │ -12.2  │ 76.8  │ 202.8  │ NA      │
│ 8   │ NA       │ -221.6  │ -81.9  │ 27.5  │ 144.5  │ NA      │
│ 9   │ NA       │ -153.7  │ -17.0  │ 76.1  │ 222.4  │ NA      │
│ 10  │ NA       │ -184.7  │ -47.3  │ 74.4  │ 208.9  │ NA      │
│ 11  │ NA       │ -191.15 │ -148.8 │ 11.4  │ 137.7  │ NA      │
│ 12  │ NA       │ -197.6  │ -6.4   │ 137.1 │ 183.6  │ NA      │
│ 13  │ NA       │ -247.8  │ -35.4  │ 80.9  │ 229.5  │ NA      │
│ 14  │ NA       │ -227.0  │ -104.7 │ 20.2  │ 140.2  │ NA      │
│ 15  │ -233.6   │ -115.9  │ -10.5  │ 70.0  │ 202.6  │ NA      │
│ 16  │ -235.6   │ -232.4  │ -100.6 │ 16.8  │ 145.1  │ NA      │
│ 17  │ -237.6   │ -199.4  │ -58.2  │ 29.1  │ 184.4  │ NA      │
│ 18  │ -239.6   │ -195.7  │ -89.5  │ 26.4  │ 142.7  │ NA      │
│ 19  │ -241.6   │ -180.1  │ -65.0  │ 27.3  │ 171.1  │ NA      │
│ 20  │ -243.6   │ -198.7  │ -85.2  │ 27.1  │ 161.3  │ NA      │
│ 21  │ -245.6   │ -217.3  │ -77.1  │ 27.6  │ 151.5  │ NA      │
│ 22  │ -247.6   │ -139.7  │ -15.8  │ 83.0  │ 215.5  │ NA      │
│ 23  │ -249.6   │ -132.8  │ -14.1  │ 78.1  │ 205.7  │ NA      │
│ 24  │ -247.86  │ -152.7  │ -36.9  │ 29.7  │ 149.8  │ NA      │
│ 25  │ -246.12  │ -224.1  │ -81.9  │ 29.1  │ 172.2  │ NA      │
│ 26  │ -244.38  │ -213.45 │ -235.8 │ 6.0   │ 144.4  │ NA      │
│ 27  │ -242.64  │ -202.8  │ -45.1  │ 84.0  │ 227.3  │ NA      │
│ 28  │ -240.9   │ -138.4  │ -21.5  │ 73.4  │ 210.6  │ NA      │
│ 29  │ -247.1   │ -128.2  │ -31.3  │ 29.2  │ 143.1  │ NA      │
│ 30  │ -247.093 │ -185.4  │ -80.3  │ 23.9  │ 115.8  │ 222.7   │
│ 31  │ -247.086 │ -182.5  │ -75.8  │ 27.5  │ 165.2  │ 232.6   │
│ 32  │ -247.079 │ -202.2  │ -99.1  │ 23.8  │ 136.3  │ 242.5   │
│ 33  │ -247.071 │ -193.3  │ -82.6  │ 26.3  │ 160.5  │ 242.082 │
│ 34  │ -247.064 │ -189.4  │ -63.3  │ 27.6  │ 136.8  │ 241.664 │
│ 35  │ -247.057 │ -149.0  │ -31.0  │ 73.5  │ 187.8  │ 241.245 │
│ 36  │ -247.05  │ -162.4  │ -26.5  │ 72.6  │ 173.15 │ 240.827 │
│ 37  │ -247.043 │ -213.4  │ -107.2 │ 24.7  │ 158.5  │ 240.409 │
⋮
│ 432 │ -219.99  │ -156.2  │ -32.9  │ 63.3  │ 182.8  │ 232.0   │
│ 433 │ -219.945 │ -220.6  │ -114.2 │ 9.7   │ 106.4  │ 227.9   │
│ 434 │ -219.9   │ -120.9  │ -1.3   │ 99.5  │ 207.6  │ 225.629 │
│ 435 │ -222.943 │ -240.5  │ -110.3 │ 26.1  │ 142.8  │ 223.357 │
│ 436 │ -225.986 │ -239.6  │ -121.4 │ 2.9   │ 124.9  │ 221.086 │
│ 437 │ -229.029 │ -139.8  │ -7.3   │ 121.0 │ 151.55 │ 218.814 │
│ 438 │ -232.071 │ -212.0  │ -66.2  │ 50.4  │ 178.2  │ 216.543 │
│ 439 │ -235.114 │ -232.7  │ -109.2 │ 18.4  │ 127.5  │ 214.271 │
│ 440 │ -238.157 │ -236.3  │ -115.1 │ 5.1   │ 109.0  │ 212.0   │
│ 441 │ -241.2   │ -107.1  │ -9.1   │ 95.1  │ 198.6  │ 216.617 │
│ 442 │ -226.7   │ -143.8  │ -30.4  │ 75.8  │ 196.6  │ 221.233 │
│ 443 │ -224.52  │ -131.8  │ -26.5  │ 64.7  │ 177.2  │ 225.85  │
│ 444 │ -222.34  │ -144.9  │ -0.9   │ 105.3 │ 230.9  │ 230.467 │
│ 445 │ -220.16  │ -214.0  │ -81.8  │ 66.1  │ 191.3  │ 235.083 │
│ 446 │ -217.98  │ -210.6  │ -94.3  │ 16.7  │ 125.5  │ 239.7   │
│ 447 │ -215.8   │ -114.8  │ -18.4  │ 65.3  │ 171.6  │ 249.7   │
│ 448 │ -216.56  │ -156.0  │ -14.0  │ 113.7 │ 249.3  │ 248.7   │
│ 449 │ -217.32  │ -210.5  │ -41.9  │ 85.25 │ 191.55 │ 247.7   │
│ 450 │ -218.08  │ -189.2  │ -72.0  │ 56.8  │ 133.8  │ 246.7   │
│ 451 │ -218.84  │ -214.2  │ -102.2 │ 5.5   │ 75.6   │ 154.3   │
│ 452 │ -219.6   │ -107.9  │ -16.0  │ 101.7 │ 186.0  │ 200.1   │
│ 453 │ -220.033 │ -153.0  │ -38.0  │ 61.3  │ 144.4  │ 245.9   │
│ 454 │ -220.467 │ -179.8  │ -63.4  │ 56.0  │ 157.5  │ 234.033 │
│ 455 │ -220.9   │ -174.5  │ -44.8  │ 73.3  │ 179.7  │ 222.167 │
│ 456 │ -221.333 │ -206.8  │ -108.9 │ 3.7   │ 102.1  │ 210.3   │
│ 457 │ -221.767 │ -169.5  │ -79.7  │ 27.9  │ 129.4  │ 242.8   │
│ 458 │ -222.2   │ -104.6  │ -2.4   │ 84.3  │ 204.7  │ 237.84  │
│ 459 │ -236.3   │ -124.0  │ -6.8   │ 95.7  │ 196.0  │ 232.88  │
│ 460 │ -237.875 │ -216.5  │ -90.2  │ 27.8  │ 138.9  │ 227.92  │
│ 461 │ -239.45  │ -163.2  │ -43.6  │ 69.5  │ 173.9  │ 222.96  │
│ 462 │ -241.025 │ -207.3  │ -88.3  │ 9.6   │ 104.1  │ 218.0   │
│ 463 │ -242.6   │ -142.0  │ -21.8  │ 69.8  │ 148.7  │ 224.125 │
│ 464 │ -235.9   │ -128.8  │ -33.1  │ 68.8  │ 177.1  │ 230.25  │
│ 465 │ -239.8   │ -140.8  │ -38.7  │ 58.1  │ 186.3  │ 236.375 │
│ 466 │ -243.7   │ -149.5  │ -40.3  │ 62.8  │ 139.7  │ 242.5   │
│ 467 │ -247.6   │ -157.8  │ -53.3  │ 28.3  │ 122.9  │ 227.6   │
│ 468 │ NA       │ -154.9  │ -50.8  │ 28.1  │ 119.9  │ 201.1   │
│ 469 │ NA       │ -180.7  │ -70.9  │ 33.7  │ 114.8  │ 222.5   │
```

Finally, we can also chain imputation methods together.
As we saw in the last example linear interpolation can interpolate missing values
at the head or tail of the array (or column).
```julia
julia> chain(df, Impute.Interpolate(), Impute.LOCF(), Impute.NOCB(); limit=1.0)
469×6 DataFrames.DataFrame
│ Row │ V1       │ V2      │ V3     │ V4    │ V5     │ V6      │
├─────┼──────────┼─────────┼────────┼───────┼────────┼─────────┤
│ 1   │ -233.6   │ -203.7  │ -84.1  │ 18.5  │ 134.7  │ 222.7   │
│ 2   │ -233.6   │ -203.0  │ -97.8  │ 25.8  │ 134.7  │ 222.7   │
│ 3   │ -233.6   │ -249.0  │ -92.1  │ 27.8  │ 177.1  │ 222.7   │
│ 4   │ -233.6   │ -231.5  │ -97.5  │ 27.0  │ 150.3  │ 222.7   │
│ 5   │ -233.6   │ -227.3  │ -130.1 │ 25.8  │ 160.0  │ 222.7   │
│ 6   │ -233.6   │ -223.1  │ -70.7  │ 62.1  │ 197.5  │ 222.7   │
│ 7   │ -233.6   │ -164.8  │ -12.2  │ 76.8  │ 202.8  │ 222.7   │
│ 8   │ -233.6   │ -221.6  │ -81.9  │ 27.5  │ 144.5  │ 222.7   │
│ 9   │ -233.6   │ -153.7  │ -17.0  │ 76.1  │ 222.4  │ 222.7   │
│ 10  │ -233.6   │ -184.7  │ -47.3  │ 74.4  │ 208.9  │ 222.7   │
│ 11  │ -233.6   │ -191.15 │ -148.8 │ 11.4  │ 137.7  │ 222.7   │
│ 12  │ -233.6   │ -197.6  │ -6.4   │ 137.1 │ 183.6  │ 222.7   │
│ 13  │ -233.6   │ -247.8  │ -35.4  │ 80.9  │ 229.5  │ 222.7   │
│ 14  │ -233.6   │ -227.0  │ -104.7 │ 20.2  │ 140.2  │ 222.7   │
│ 15  │ -233.6   │ -115.9  │ -10.5  │ 70.0  │ 202.6  │ 222.7   │
│ 16  │ -235.6   │ -232.4  │ -100.6 │ 16.8  │ 145.1  │ 222.7   │
│ 17  │ -237.6   │ -199.4  │ -58.2  │ 29.1  │ 184.4  │ 222.7   │
│ 18  │ -239.6   │ -195.7  │ -89.5  │ 26.4  │ 142.7  │ 222.7   │
│ 19  │ -241.6   │ -180.1  │ -65.0  │ 27.3  │ 171.1  │ 222.7   │
│ 20  │ -243.6   │ -198.7  │ -85.2  │ 27.1  │ 161.3  │ 222.7   │
│ 21  │ -245.6   │ -217.3  │ -77.1  │ 27.6  │ 151.5  │ 222.7   │
│ 22  │ -247.6   │ -139.7  │ -15.8  │ 83.0  │ 215.5  │ 222.7   │
│ 23  │ -249.6   │ -132.8  │ -14.1  │ 78.1  │ 205.7  │ 222.7   │
│ 24  │ -247.86  │ -152.7  │ -36.9  │ 29.7  │ 149.8  │ 222.7   │
│ 25  │ -246.12  │ -224.1  │ -81.9  │ 29.1  │ 172.2  │ 222.7   │
│ 26  │ -244.38  │ -213.45 │ -235.8 │ 6.0   │ 144.4  │ 222.7   │
│ 27  │ -242.64  │ -202.8  │ -45.1  │ 84.0  │ 227.3  │ 222.7   │
│ 28  │ -240.9   │ -138.4  │ -21.5  │ 73.4  │ 210.6  │ 222.7   │
│ 29  │ -247.1   │ -128.2  │ -31.3  │ 29.2  │ 143.1  │ 222.7   │
│ 30  │ -247.093 │ -185.4  │ -80.3  │ 23.9  │ 115.8  │ 222.7   │
│ 31  │ -247.086 │ -182.5  │ -75.8  │ 27.5  │ 165.2  │ 232.6   │
│ 32  │ -247.079 │ -202.2  │ -99.1  │ 23.8  │ 136.3  │ 242.5   │
│ 33  │ -247.071 │ -193.3  │ -82.6  │ 26.3  │ 160.5  │ 242.082 │
│ 34  │ -247.064 │ -189.4  │ -63.3  │ 27.6  │ 136.8  │ 241.664 │
│ 35  │ -247.057 │ -149.0  │ -31.0  │ 73.5  │ 187.8  │ 241.245 │
│ 36  │ -247.05  │ -162.4  │ -26.5  │ 72.6  │ 173.15 │ 240.827 │
│ 37  │ -247.043 │ -213.4  │ -107.2 │ 24.7  │ 158.5  │ 240.409 │
⋮
│ 432 │ -219.99  │ -156.2  │ -32.9  │ 63.3  │ 182.8  │ 232.0   │
│ 433 │ -219.945 │ -220.6  │ -114.2 │ 9.7   │ 106.4  │ 227.9   │
│ 434 │ -219.9   │ -120.9  │ -1.3   │ 99.5  │ 207.6  │ 225.629 │
│ 435 │ -222.943 │ -240.5  │ -110.3 │ 26.1  │ 142.8  │ 223.357 │
│ 436 │ -225.986 │ -239.6  │ -121.4 │ 2.9   │ 124.9  │ 221.086 │
│ 437 │ -229.029 │ -139.8  │ -7.3   │ 121.0 │ 151.55 │ 218.814 │
│ 438 │ -232.071 │ -212.0  │ -66.2  │ 50.4  │ 178.2  │ 216.543 │
│ 439 │ -235.114 │ -232.7  │ -109.2 │ 18.4  │ 127.5  │ 214.271 │
│ 440 │ -238.157 │ -236.3  │ -115.1 │ 5.1   │ 109.0  │ 212.0   │
│ 441 │ -241.2   │ -107.1  │ -9.1   │ 95.1  │ 198.6  │ 216.617 │
│ 442 │ -226.7   │ -143.8  │ -30.4  │ 75.8  │ 196.6  │ 221.233 │
│ 443 │ -224.52  │ -131.8  │ -26.5  │ 64.7  │ 177.2  │ 225.85  │
│ 444 │ -222.34  │ -144.9  │ -0.9   │ 105.3 │ 230.9  │ 230.467 │
│ 445 │ -220.16  │ -214.0  │ -81.8  │ 66.1  │ 191.3  │ 235.083 │
│ 446 │ -217.98  │ -210.6  │ -94.3  │ 16.7  │ 125.5  │ 239.7   │
│ 447 │ -215.8   │ -114.8  │ -18.4  │ 65.3  │ 171.6  │ 249.7   │
│ 448 │ -216.56  │ -156.0  │ -14.0  │ 113.7 │ 249.3  │ 248.7   │
│ 449 │ -217.32  │ -210.5  │ -41.9  │ 85.25 │ 191.55 │ 247.7   │
│ 450 │ -218.08  │ -189.2  │ -72.0  │ 56.8  │ 133.8  │ 246.7   │
│ 451 │ -218.84  │ -214.2  │ -102.2 │ 5.5   │ 75.6   │ 154.3   │
│ 452 │ -219.6   │ -107.9  │ -16.0  │ 101.7 │ 186.0  │ 200.1   │
│ 453 │ -220.033 │ -153.0  │ -38.0  │ 61.3  │ 144.4  │ 245.9   │
│ 454 │ -220.467 │ -179.8  │ -63.4  │ 56.0  │ 157.5  │ 234.033 │
│ 455 │ -220.9   │ -174.5  │ -44.8  │ 73.3  │ 179.7  │ 222.167 │
│ 456 │ -221.333 │ -206.8  │ -108.9 │ 3.7   │ 102.1  │ 210.3   │
│ 457 │ -221.767 │ -169.5  │ -79.7  │ 27.9  │ 129.4  │ 242.8   │
│ 458 │ -222.2   │ -104.6  │ -2.4   │ 84.3  │ 204.7  │ 237.84  │
│ 459 │ -236.3   │ -124.0  │ -6.8   │ 95.7  │ 196.0  │ 232.88  │
│ 460 │ -237.875 │ -216.5  │ -90.2  │ 27.8  │ 138.9  │ 227.92  │
│ 461 │ -239.45  │ -163.2  │ -43.6  │ 69.5  │ 173.9  │ 222.96  │
│ 462 │ -241.025 │ -207.3  │ -88.3  │ 9.6   │ 104.1  │ 218.0   │
│ 463 │ -242.6   │ -142.0  │ -21.8  │ 69.8  │ 148.7  │ 224.125 │
│ 464 │ -235.9   │ -128.8  │ -33.1  │ 68.8  │ 177.1  │ 230.25  │
│ 465 │ -239.8   │ -140.8  │ -38.7  │ 58.1  │ 186.3  │ 236.375 │
│ 466 │ -243.7   │ -149.5  │ -40.3  │ 62.8  │ 139.7  │ 242.5   │
│ 467 │ -247.6   │ -157.8  │ -53.3  │ 28.3  │ 122.9  │ 227.6   │
│ 468 │ -247.6   │ -154.9  │ -50.8  │ 28.1  │ 119.9  │ 201.1   │
│ 469 │ -247.6   │ -180.7  │ -70.9  │ 33.7  │ 114.8  │ 222.5   │
```
